@@ -2110,17 +2110,21 @@ e_book_backend_exchange_build_restriction (const char *query,
 	r = e_sexp_eval (sexp);
 	if (r && r->type == ESEXP_RES_UNDEFINED)
 		rn = (E2kRestriction *)r->value.string;
-	else {
-		g_warning ("conversion to exchange restriction failed");
+	else
 		rn = NULL;
-	}
+
+	if (!rn)
+		g_warning ("conversion to exchange restriction failed, query: '%s'", query ? query : "[null]");
 
 	e_sexp_result_free (sexp, r);
 	e_sexp_unref (sexp);
 
-	if (base_rn) {
+	if (base_rn && rn) {
 		e2k_restriction_ref (base_rn);
 		rn = e2k_restriction_andv (rn, base_rn, NULL);
+
+		if (!rn)
+			g_warning ("failed to concat with a base_rn, query: '%s'", query ? query : "[null]");
 	}
 
 	return rn;
@@ -2234,9 +2238,14 @@ e_book_backend_exchange_get_contact_list (EBookBackendSync  *backend,
 
 		rn = e_book_backend_exchange_build_restriction (query,
 								bepriv->base_rn);
+
+		if (!rn)
+			return GNOME_Evolution_Addressbook_OtherError;
+
 		iter = e_folder_exchange_search_start (bepriv->folder, NULL,
 					       field_names, n_field_names,
 					       rn, NULL, TRUE);
+
 		e2k_restriction_unref (rn);
 
 		*contacts = NULL;
@@ -2330,6 +2339,9 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 		rn = e_book_backend_exchange_build_restriction (query,
 							bepriv->base_rn);
 
+		if (!rn)
+			return;
+
 		iter = e_folder_exchange_search_start (bepriv->folder, NULL,
 					       field_names, n_field_names,
 					       rn, NULL, TRUE);
@@ -2396,7 +2408,7 @@ find_deleted_ids (const char *id, const char *vcard, gpointer user_data)
 		char *vcard = NULL;
 		EContact *contact = e_contact_new ();
 		if (contact) {
-			e_contact_set (contact, E_CONTACT_UID, id);
+			e_contact_set (contact, E_CONTACT_UID, (gpointer) id);
 			vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
 			if (vcard) {
 				ctx->changes = g_list_prepend (
